/* Header for module k196_con, generated by p2c */
#ifndef K196_CON_H
#define K196_CON_H


#ifdef K196_CON_G
# define vextern
#else
# define vextern extern
#endif


extern boolean K196_Error PV();
extern Void trigger_K196 PV();
extern Void Init_K196 PV();
extern Void Set_k196_Res PP((long res));
extern Void Set_k196_Range PP((long range));
extern Void Set_k196_Mode PP((int M, int T));
extern Void Set_k196_Filter PP((long filter));
extern double Read_k196 PV();
/*  Reads Keithley 196 DVM  */
extern Void Reset_k196 PV();


#undef vextern

#endif /*K196_CON_H*/

/* End. */
