/* Header for module viewctrl, generated by p2c */
#ifndef VIEWCTRL_H
#define VIEWCTRL_H


#ifdef VIEWCTRL_G
# define vextern
#else
# define vextern extern
#endif




extern Void viewctrl_viewbuiltin PV();


#undef vextern

#endif /*VIEWCTRL_H*/

/* End. */
