/* Header for module viewconf, generated by p2c */
#ifndef VIEWCONF_H
#define VIEWCONF_H


#ifdef VIEWCONF_G
# define vextern
#else
# define vextern extern
#endif


#define v_version       "1.14"

/* The following file should assign v_hasdebug, v_hashelp */

#define v_hashelp       true
#define v_hasdebug      false


#undef vextern

#endif /*VIEWCONF_H*/

/* End. */
