/* Header for module MMMT, generated by p2c */
#ifndef MMMT_H
#define MMMT_H


/*K196_con,*/
/*library for AC voltmeter*/

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef VIEWMOD_H
#include "../viewmod.h"
#endif


#ifdef MMMT_G
# define vextern
#else
# define vextern extern
#endif

extern Void MT_MM_MAIN PP((mt_action action_));

#undef vextern

#endif /*MMMT_H*/

/* End. */
