/* Header for module FCMT, generated by p2c */
#ifndef FCMT_H
#define FCMT_H

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef VIEWMOD_H
#include "viewmod.h"
#endif


#ifdef FCMT_G
# define vextern
#else
# define vextern extern
#endif


extern Void MT_FC_MAIN PP((mt_action action_));

#undef vextern

#endif /*FCMT_H*/

/* End. */
