/* Header for module viewfuncs, generated by p2c */
#ifndef VIEWFUNCS_H
#define VIEWFUNCS_H


#ifdef VIEWFUNCS_G
# define vextern
#else
# define vextern extern
#endif




extern Void viewfuncs_viewbuiltin PV();


#undef vextern

#endif /*VIEWFUNCS_H*/

/* End. */
