/* Header for module viewcmds, generated by p2c */
#ifndef VIEWCMDS_H
#define VIEWCMDS_H


#ifdef VIEWCMDS_G
# define vextern
#else
# define vextern extern
#endif




extern Void viewcmds_viewbuiltin PV();


#undef vextern

#endif /*VIEWCMDS_H*/

/* End. */
