/* Header for module FGMT, generated by p2c */
#ifndef FGMT_H
#define FGMT_H

/*library for frequency generator*/

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef NEWCI_H
#include <p2c/newci.h>
#endif

#ifndef VIEWMOD_H
#include "viewmod.h"
#endif


#ifdef FGMT_G
# define vextern
#else
# define vextern extern
#endif

extern Void MT_FG_MAIN PP((mt_action action_));


#undef vextern

#endif /*FGMT_H*/

/* End. */
