/* Header for module MT, generated by p2c */
#ifndef MT_H
#define MT_H


#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef NEWCI_H
#include <p2c/newci.h>
#endif


#ifndef VIEWMOD_H
#include "../viewmod.h"
#endif


#ifdef MT_G
# define vextern
#else
# define vextern extern
#endif

#include "/home/auspex/f/sounds/lazzaro/chipmunk/iotech/new/sun4c/4.1.1c/ieeeio.h"


/*actions a main procedure may be called with*/

/*resets your state variables to sane values*/
/*initialize your library*/
/*initialize your library*/
/*update your physical device*/
/*make your appropriate view parameters*/
typedef enum {
  mt_reset, mt_init, mt_uninit, mt_update, mt_makeparam
} mt_action;


extern Void MT_viewinit PV();
/*initializes viewtool*/
extern Void MT_exit PV();
/*does mytools-specific stuff on exit*/


#undef vextern

#endif /*MT_H*/

/* End. */
