/* Header for module viewcurves, generated by p2c */
#ifndef VIEWCURVES_H
#define VIEWCURVES_H


#ifdef VIEWCURVES_G
# define vextern
#else
# define vextern extern
#endif


extern Void curves_viewinit PV();


vextern Void (*(curves_tool_list[2]))()
#ifdef VIEWCURVES_G
  = {
  curves_viewinit,
  NULL
}
#endif VIEWCURVES_G
  ;



#undef vextern

#endif /*VIEWCURVES_H*/

/* End. */
