/* Header for module take2430, generated by p2c */
#ifndef TAKE2430_H
#define TAKE2430_H



#ifndef VIEWMOD_H
#include "../viewmod.h"
#endif

#ifndef TEK2430_H
#include "tek2430.h"
#endif

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef NEWCI_H
#include <p2c/newci.h>
#endif


#ifdef TAKE2430_G
# define vextern
#else
# define vextern extern
#endif



extern Void take2430_viewinit PV();


#undef vextern

#endif /*TAKE2430_H*/

/* End. */
